/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef PUZZLE_H
#define PUZZLE_H

#include <QLineEdit>
#include <QMouseEvent>

class Puzzle : public QWidget {
	Q_OBJECT
private:
	class Ledit : public QLineEdit {
	public:
		Ledit(QWidget *p = 0) : QLineEdit(p) {}
	private:
		virtual void 
		mousePressEvent(QMouseEvent *ev)
		{
			selectAll();
			ev->accept();
		}
	};
public:
	Puzzle(QWidget *p = 0);
	~Puzzle();

	void clear() { for (int i = 0; i < 7; i++) le[i]->clear(); }
	void setReadOnly(bool r); 
	void setFont(QFont & f) { for (int i = 0; i < 7; i++) le[i]->setFont(f); }
	Ledit* centralLetr() const { return le[6]; }
	void setLetrs(QString s);
	QString letrs() const;
private slots:
	void toUpper(); 
private:
	virtual void paintEvent(QPaintEvent *ev);
	virtual void mousePressEvent(QMouseEvent *ev);
private:
	double			sd, sd2, tx, ty;
	Ledit			*le[7];
	QPolygon		*p1, *p2, *p3, *p4, *p5, *p6, *p7;
	QPointF			points[6], points2[6];
	Puzzle(const Puzzle &);
	Puzzle & operator=(const Puzzle &);
};

#endif /* PUZZLE_H */
