/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QDialog>
#include <QShowEvent>
#include <QFont>
#include <QMessageBox>
#include <QFileDialog>
#include <QFontDialog>

#include "settings.h"
#include "settingspersist.h"
#include "../logic/defines.h"

Settings::Settings(QWidget *p, SettingsPersist *s) 
	: QDialog(p), settings_(s) 
{ 	
	setupUi(this); 
	connect(this, SIGNAL(accepted()), SLOT(settingsAccept()));
	connect(dictSelectButton, SIGNAL(clicked()), this, SLOT(changeDict()));
	connect(monoFontButton, SIGNAL(clicked()), this, SLOT(changeFontMono()));
	connect(spinBox_max, SIGNAL(valueChanged(int)), this, SLOT(adjustSbMin(int)));
	connect(spinBox_min, SIGNAL(valueChanged(int)), this, SLOT(adjustSbMax(int)));
	spinBox_max->setRange(MMIN_SOL + MGAP_SOL, MMAX_SOL);
	spinBox_min->setRange(MMIN_SOL, MMAX_SOL - MGAP_SOL);
}

void
Settings::showEvent(QShowEvent *ev)
{ 
	spinBox_min->setValue(settings_->minSol);
	spinBox_max->setValue(settings_->maxSol);
	dictPathEdit->setText(settings_->dictPath);
	precompCheckBox->setChecked(settings_->dictPrecomp);
	scoreCheckBox->setChecked(settings_->showScore);
	timeCheckBox->setChecked(settings_->showTime);
	pluralCheckBox->setChecked(settings_->pluralSieve);
	font = settings_->monoFont;
	monoFontLineEdit->setFont(font);
	monoFontLineEdit->setText(QString("%1, %2pt").arg(font.family())
								.arg(font.pointSize()));
	ev->accept();
}

void
Settings::settingsAccept()
{
	settings_->minSol = spinBox_min->value();
	settings_->maxSol = spinBox_max->value();
	settings_->showScore = scoreCheckBox->isChecked();
	settings_->showTime = timeCheckBox->isChecked();
	settings_->pluralSieve = pluralCheckBox->isChecked();
	settings_->monoFont = font;
	settings_->dictPath = dictPathEdit->text();
	settings_->dictPrecomp = precompCheckBox->isChecked();
	if (!settings_->save())
		QMessageBox::critical(this, "Error", "Error while writing settings to disc.");
	emit settingsAccepted();
}

void 
Settings::adjustSbMin(int v)
{
	if (v - spinBox_min->value() >= MGAP_SOL)
		spinBox_min->setMaximum(v - MGAP_SOL);
}

void 
Settings::adjustSbMax(int v)
{
	if (spinBox_max->value() - v >= MGAP_SOL)
		spinBox_max->setMinimum(v + MGAP_SOL);
}

void
Settings::changeDict()
{
	QString dpath = QFileDialog::getOpenFileName(this, "Choose Dictionary", 
												 settings_->dictPath);

	if (dpath != "") 
		dictPathEdit->setText(dpath);
}

void
Settings::changeFontMono()
{
	bool ok;

	QFont f = QFontDialog::getFont(&ok, settings_->monoFont, this);
	if (ok) {
		font = f;
		monoFontLineEdit->setFont(f);
		monoFontLineEdit->setText(QString("%1, %2pt").arg(f.family()).arg(f.pointSize()));
	}
}


