/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef DICT_H
#define DICT_H

#include <vector>
#include <string>

#ifndef SPTH_CL
#  include <QObject>
#  include <QWidget>
#endif /* !SPTH_CL */

#include "defines.h"

using std::vector;
using std::string;

namespace Spellathon
{
	
	struct dict_ent {	
		char sign[8];
		char word[8];
	};

	struct precomp_ent {
		char sign[8];
		int  nsol;
	};
	
#ifdef SPTH_CL
	class Dict {
	public:
		Dict(); 
#else		
	class Dict : public QObject {
		Q_OBJECT
	public:
		Dict(QWidget *p = 0); 
#endif /* SPTH_CL */
		~Dict() { destroy(); }

		void destroy();
		void init(const char *path, bool precomp = PRECOMP, 
				  bool plural_sieve = PLURAL_SIEVE);

		void matchSign(const char *sign, vector<char*> &w) const;
		bool matchWord(const char *word) const;

		int		off7() const { return off[7].start; } 
		int		nent7() const { return off[7].end - off[7].start + 1; } 
		int		precomp_nent() const { return precomp_hdr.precomp_nent; }
		bool	precomp() const { return precomp_; }
		string	dictPath() const { return dictPath_; }
		bool	pluralSieve() const { return plural_sieve_; }

	public:
		dict_ent	*dictarr;	
		precomp_ent	*precomparr;	

	private:
		inline int bsearch(const char * sign, int slen) const;
		void	dictOpen(const char *path);
		void	precompOpen(const char *path);
		void	precompCreate(const char *path);
		void	setPrecompPath();

	private:
		enum {MAX_LNLEN = 64, MAX_PATH_LEN = 1024};

		char	(*words)[8];	/* list of all words of 3 <= strlen() <=7	 */
								/* the 3 letter words are needed for finding */
								/* plurals.						 			 */

		struct off_st {
			int start;	
			int end; 
		} off[8];	/* dict. offsets. arr index = strlen */

		struct precomp_hdr_st {
			char	magic[16];
			int		dict_nent;	
			int		precomp_nent;
			int		plural_sieve;
			struct off_st off[8];
			char	path[MAX_PATH_LEN];
		} precomp_hdr;

		int		nent_;	/* total words */
		bool	precomp_;
		bool	plural_sieve_;
		char	dictbin_[MAX_PATH_LEN];
		string	dictPath_;
#ifndef SPTH_CL
		QWidget	*gui_h_;
#endif /* SPTH_CL */
		Dict(const Dict &);
		Dict & operator=(const Dict &);
	};
	
}

#endif /* DICT_H */
